@@ -1,175 +1,14 @@
-use std::collections::BTreeMap;
-use std::thread;
+use getopts::Options;
 #[cfg(not(target_os = "linux"))]
 use libc::*;
+use std::collections::BTreeMap;
 #[cfg(not(target_os = "linux"))]
 use std::ffi;
+use std::thread;
 
-struct Device {
-    // TODO: this should be a structure of counters, perhaps holding all the proc data
-    name: String,
+use chrono::{DateTime, Local, Timelike};
 
-    ibytes: i64,
-    obytes: i64,
-    ipackets: i64,
-    opackets: i64,
-}
-
-struct DeviceHist {
-    history: Vec<Device>,
-}
-
-#[cfg(target_os = "linux")]
-enum ProcNetDevIndexes {
-    Ibytes = 1,
-    Ipackets = 2,
-    Obytes = 9,
-    Opackets = 10,
-}
-
-#[cfg(target_os = "linux")]
-fn line_to_device_struct(line: &str) -> Result<Device, ()> {
-    let idx = line.find(':');
-    if idx == None {
-        return Err(());
-    }
-
-    let part_split = line[idx.unwrap()..].split_ascii_whitespace();
-    let vec_list = part_split.collect::<Vec<&str>>();
-
-    let d = Device {
-        name: (line[0..idx.unwrap()].trim()).to_string(),
-        ibytes: vec_list[ProcNetDevIndexes::Ibytes as usize]
-            .parse()
-            .unwrap_or(0),
-        obytes: vec_list[ProcNetDevIndexes::Obytes as usize]
-            .parse()
-            .unwrap_or(0),
-        ipackets: vec_list[ProcNetDevIndexes::Ipackets as usize]
-            .parse()
-            .unwrap_or(0),
-        opackets: vec_list[ProcNetDevIndexes::Opackets as usize]
-            .parse()
-            .unwrap_or(0),
-    };
-    Ok(d)
-}
-
-#[cfg(target_os = "linux")]
-fn read_net() -> Vec<Device> {
-    use std::fs::File;
-    use std::io::{prelude::*, BufReader};
-
-    let mut devs = Vec::new();
-    let file = File::open("/proc/net/dev").unwrap();
-    let reader = BufReader::new(file);
-    for (_, line) in reader.lines().enumerate() {
-        let a = line.unwrap();
-        let d = line_to_device_struct(&a);
-        match d {
-            Err(_x) => {}
-            Ok(d) => {
-                devs.push(d);
-            }
-        }
-    }
-
-    devs
-}
-
-pub type Int64T = ::std::os::raw::c_long;
-pub type Uint64T = ::std::os::raw::c_ulong;
-pub type UChar = ::std::os::raw::c_uchar;
-pub type TimeT = Int64T;
-
-#[repr(C)]
-#[derive(Debug, Copy, Clone)]
-pub struct timespec {
-    pub tv_sec: TimeT,
-    pub tv_nsec: ::std::os::raw::c_long,
-}
-
-#[repr(C)]
-#[derive(Debug, Copy, Clone)]
-pub struct rtnl_link_stats {
-    pub ifi_type: UChar,
-    pub ifi_addrlen: UChar,
-    pub ifi_hdrlen: UChar,
-    pub ifi_link_state: ::std::os::raw::c_int,
-    pub ifi_mtu: Uint64T,
-    pub ifi_metric: Uint64T,
-    pub ifi_baudrate: Uint64T,
-    pub ifi_ipackets: Uint64T,
-    pub ifi_ierrors: Uint64T,
-    pub ifi_opackets: Uint64T,
-    pub ifi_oerrors: Uint64T,
-    pub ifi_collisions: Uint64T,
-    pub ifi_ibytes: Uint64T,
-    pub ifi_obytes: Uint64T,
-    pub ifi_imcasts: Uint64T,
-    pub ifi_omcasts: Uint64T,
-    pub ifi_iqdrops: Uint64T,
-    pub ifi_noproto: Uint64T,
-    pub ifi_lastchange: timespec,
-}
-
-#[cfg(not(target_os = "linux"))]
-fn read_net() -> Vec<Device> {
-    let mut devs = Vec::new();
-
-    unsafe {
-        let mut addrs: *mut libc::ifaddrs = std::mem::MaybeUninit::uninit().as_mut_ptr();
-        let _a = libc::getifaddrs( &mut addrs );
-
-        loop {
-            if (*(*addrs).ifa_addr).sa_family as i32 == libc::AF_LINK {
-                let s:*const rtnl_link_stats  = ((*addrs).ifa_data) as *const rtnl_link_stats;
-                let d = Device {
-                    name: ffi::CStr::from_ptr((*addrs).ifa_name).to_str().unwrap().to_string(),
-                    ibytes: (*s).ifi_ibytes as i64,
-                    obytes: (*s).ifi_obytes as i64,
-                    ipackets: (*s).ifi_ipackets as i64,
-                    opackets: (*s).ifi_opackets as i64,
-                };
-
-                devs.push(d);
-            }
-
-            addrs = (*addrs).ifa_next;
-
-            if std::ptr::null() == addrs {
-                freeifaddrs( addrs );
-                break;
-            }
-        }
-    };
-
-    devs
-}
-
-fn human_unit(value: i64) -> String {
-    let kb = 1024;
-    let mb = 1024 * 1024;
-    let gb = 1024 * 1024 * 1024;
-    let tb = 1024 * 1024 * 1024 * 1024;
-    let pb = 1024 * 1024 * 1024 * 1024 * 1024;
-    if value >= pb {
-        return format!("{} PiB", value / pb);
-    }
-    if value >= tb {
-        return format!("{} TiB", value / tb);
-    }
-    if value >= gb {
-        return format!("{} GiB", value / gb);
-    }
-    if value >= mb {
-        return format!("{} MiB", value / mb);
-    }
-    if value >= kb {
-        return format!("{} KiB", value / kb);
-    }
-    value.to_string()
-}
+use netr::*;
 
 fn print_line_stats(
     i: &str,
@@ -185,10 +24,10 @@ fn print_line_stats(
         i,
         human_unit(in_total),
         human_unit(out_total),
-        human_unit(isec as i64),
-        human_unit(osec as i64),
-        human_unit(imin as i64),
-        human_unit(omin as i64),
+        human_unit(isec),
+        human_unit(osec),
+        human_unit(imin),
+        human_unit(omin),
     );
 }
 
@@ -228,23 +67,29 @@ fn print_stats(total: &str, h: &BTreeMap<String, DeviceHist>) {
         };
 
         let imin = if history_len > 1 {
-            dl.history[history_len - 1].ibytes - dl.history[0].ibytes as i64
+            dl.history[history_len - 1].ibytes - dl.history[0].ibytes
         } else {
-            0 as i64
+            0_i64
         };
 
         let omin = if history_len > 1 {
-            dl.history[history_len - 1].obytes - dl.history[0].obytes as i64
+            dl.history[history_len - 1].obytes - dl.history[0].obytes
         } else {
-            0 as i64
+            0_i64
         };
 
-        print_line_stats(&k, in_total, out_total, isec, osec, imin, omin);
+        print_line_stats(k, in_total, out_total, isec, osec, imin, omin);
     }
     println!();
 }
 
-fn print_graph(total: &str, h: &BTreeMap<String, DeviceHist>, height: i32) {
+fn print_graph(
+    total: &str,
+    h: &BTreeMap<String, DeviceHist>,
+    height: i32,
+    indent: usize,
+    print_max: bool,
+) {
     if !h.contains_key(total) {
         return;
     }
@@ -267,7 +112,7 @@ fn print_graph(total: &str, h: &BTreeMap<String, DeviceHist>, height: i32) {
 
     // iterate lines as we draw down
     for line in 1..height {
-        print!("{:10}", "");
+        print!("{:indent$}", "", indent = indent);
         for hist in 1..history_list.len() {
             let last_total = history_list[hist - 1].ibytes + history_list[hist - 1].obytes;
             let last_in_diff = history_list[hist].ibytes - history_list[hist - 1].ibytes;
@@ -287,12 +132,43 @@ fn print_graph(total: &str, h: &BTreeMap<String, DeviceHist>, height: i32) {
                 }
             );
         }
+
+        if line == 1 && print_max {
+            print!(
+                "{:padding$} {:>9}",
+                "",
+                human_unit(max_delta),
+                padding = 61 - history_list.len()
+            );
+        }
+
         println!();
     }
 }
 
-fn run_loops() {
+fn print_minute_marker(
+    total: &str,
+    h: &BTreeMap<String, DeviceHist>,
+    time: chrono::DateTime<Local>,
+    indent: usize,
+) {
+    let padding = h.get(total).unwrap().history.len() as u32;
+    if padding > time.second() {
+        let padding = padding - time.second() - 2;
+        println!(
+            "{:indent$}{:padding$}^ {:02}:{:02}",
+            "",
+            "",
+            time.hour(),
+            time.minute(),
+            padding = padding as usize
+        );
+    }
+}
+
+fn run_loops(config: &Config) {
     let total = "total";
+    let indent = 10;
     let mut ifs: BTreeMap<String, DeviceHist> = BTreeMap::new();
 
     loop {
@@ -315,6 +191,13 @@ fn run_loops() {
         let mut opackets: i64 = 0;
 
         for dev in devs.iter() {
+            if config.filter.is_some() {
+                let c = config.clone().filter.unwrap();
+                if !c.matches(&dev.name) {
+                    continue;
+                }
+            }
+
             if !ifs.contains_key(&dev.name) {
                 ifs.insert(
                     dev.name.to_string(),
@@ -335,7 +218,7 @@ fn run_loops() {
                 opackets: dev.opackets,
             });
             dh.insert(dev.name.to_string(), true);
-            while hh.len() > 60 {
+            while hh.len() > 61 {
                 hh.remove(0);
             }
 
@@ -355,7 +238,7 @@ fn run_loops() {
             ipackets,
             opackets,
         });
-        while total_history.len() > 60 {
+        while total_history.len() > 61 {
             total_history.remove(0);
         }
         dh.insert(total.to_string(), true);
@@ -371,7 +254,10 @@ fn run_loops() {
         }
 
         print_stats(total, &ifs);
-        print_graph(total, &ifs, 10);
+        print_graph(total, &ifs, 10, indent, true);
+
+        let time: DateTime<Local> = Local::now();
+        print_minute_marker(total, &ifs, time, indent);
 
         loop {
             thread::sleep(std::time::Duration::from_millis(100));
@@ -381,17 +267,83 @@ fn run_loops() {
         }
     }
 }
+fn banner() -> String {
+    format!(
+        "{} version {}",
+        env!("CARGO_PKG_NAME"),
+        env!("CARGO_PKG_VERSION")
+    )
+}
+
+fn print_version() {
+    println!("{}", &banner());
+}
+
+fn help(opts: &getopts::Options) {
+    println!("{}", opts.usage(&banner()));
+}
 
 fn main() {
-    run_loops();
-}
+    let args: Vec<String> = std::env::args().collect();
+    let mut config = Config::new();
+    let mut opts = Options::new();
+    opts.parsing_style(getopts::ParsingStyle::FloatingFrees);
+    opts.optopt("e", "exclude", "exclude this pattern", "REGEX");
+    opts.optopt("i", "include", "include this pattern", "REGEX");
+    opts.optflag("h", "help", "display help");
+    opts.optflag("v", "version", "display version");
 
-#[test]
-fn test_human_units() {
-    assert_eq!(human_unit(1024), "1 KiB");
-    assert_eq!(human_unit(1024 * 2), "2 KiB");
-    assert_eq!(human_unit(1024 * 1024), "1 MiB");
-    assert_eq!(human_unit(1024 * 1024 * 1024), "1 GiB");
-    assert_eq!(human_unit(1024 * 1024 * 1024 * 1024), "1 TiB");
-    assert_eq!(human_unit(1024 * 1024 * 1024 * 1024 * 1024), "1 PiB");
+    let matches = match opts.parse(&args[1..]) {
+        Ok(m) => m,
+        Err(f) => {
+            println!("{}", f);
+            std::process::exit(1);
+        }
+    };
+
+    if matches.opt_present("version") {
+        print_version();
+        std::process::exit(0);
+    }
+
+    if matches.opt_present("help") {
+        help(&opts);
+        std::process::exit(0);
+    }
+
+    if matches.opt_present("exclude") {
+        let pattern = matches.opt_str("exclude").unwrap();
+        let filter = match FilterOpts::create(&pattern, !matches.opt_present("exclude")) {
+            Some(x) => x,
+            None => {
+                eprintln!("Cannot create regex from {}", pattern);
+                help(&opts);
+                std::process::exit(1);
+            }
+        };
+
+        config.filter = Some(filter);
+    }
+
+    if matches.opt_present("include") {
+        if matches.opt_present("exclude") {
+            eprintln!("Cannot --exclude and --include at the same time");
+            help(&opts);
+            std::process::exit(1);
+        }
+
+        let pattern = matches.opt_str("include").unwrap();
+        let filter = match FilterOpts::create(&pattern, matches.opt_present("include")) {
+            Some(x) => x,
+            None => {
+                eprintln!("Cannot create regex from {}", matches.free[0].clone());
+                help(&opts);
+                std::process::exit(1);
+            }
+        };
+
+        config.filter = Some(filter);
+    }
+
+    run_loops(&config);
 }
